package immibis.core.net;

import immibis.core.CoreProxy;
import immibis.core.NonSharedProxy;
import immibis.core.porting.PortableContainer;

import java.util.List;
import net.minecraft.src.*;
import net.minecraft.src.forge.MinecraftForge;

public abstract class AbstractSyncedContainer extends PortableContainer implements ISyncedContainer {
	public AbstractSyncedContainer(EntityPlayer player, IInventory inv) {
		super(player, inv);
	}

	/** Sends the packet to all players listening to this inventory.
	 * Use this to update the info displayed in the GUI, for example.
	 * They may be sent from the server to the client.
	 * 
	 * Calling this on the client has no effect in SSP, and throws
	 * an exception in SMP. It does NOT send the packet from the
	 * client to itself, on the rationale that the container should
	 * already know the information being sent.
	 */
	public void sendUpdatePacket(IPacket packet) {
		if(NonSharedProxy.CLIENT) {
			if(NonSharedProxy.isSSP())
				return;
			else
				throw new IllegalStateException("Cannot send update packets from the client");
		}
		String channel = getChannel();
		for(ICrafting ic : (List<ICrafting>)crafters)
			if(ic instanceof EntityPlayer)
				OneTwoFiveNetworking.send(channel, packet, (EntityPlayer)ic);
	}
	
	/** Sends the packet to the server. Throws an exception if run on the server.
	 * They may be sent from the client to the server, or the client
	 * to itself in SSP.
	 */
	public void sendActionPacket(IPacket packet) {
		if(NonSharedProxy.SERVER)
			throw new IllegalStateException("Cannot send action packets from the server");
		if(NonSharedProxy.isSSP())
			onActionPacket(packet);
		else
			OneTwoFiveNetworking.send(getChannel(), packet, null);
	}
	
	/** Must return the channel to send container packets on. */
	public abstract String getChannel();
	
	/** Called when an action packet is received. */
	public void onActionPacket(IPacket p) {}
	/** Called when an update packet is received. */
	public void onUpdatePacket(IPacket p) {}
	
	/** Called when a button-press packet is received.
	 * It's like an action packet that carries a single int,
	 * for convenience (you don't have to make a packet
	 * class to wrap just one int)
	 */
	public void onButtonPressed(int id) {}
	
	/** Sends a button-press packet. */
	public void sendButtonPressed(int id) {
		if(NonSharedProxy.SERVER)
			throw new IllegalStateException("Cannot send action packets from the server");
		if(NonSharedProxy.isSSP())
			onButtonPressed(id);
		else
			OneTwoFiveNetworking.send(CoreProxy.CHANNEL_GUI, new PacketButtonPress(id), null);
	}
	
	public final void onReceivePacket(IPacket p) {
		if(NonSharedProxy.SERVER) {
			if(p instanceof PacketButtonPress)
				onButtonPressed(((PacketButtonPress)p).buttonID);
			else
				onActionPacket(p);
		} else
			onUpdatePacket(p);
	}
}
